<?php

namespace Icinga\Module\Director\Clicommands;

use Icinga\Module\Director\Cli\ObjectCommand;

/**
 * Manage Icinga Usergroups
 *
 * Use this command to show, create, modify or delete Icinga Usergroup
 * objects
 */
class UsergroupCommand extends ObjectCommand
{
}
